package org.prelle.shell;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

import org.prelle.user.ConfigUser;

/**
 * ConnectedUser.java
 *
 *
 * Created: Fri Aug  2 14:39:48 2002
 *
 * @author Stefan Prelle
 * @version $Id$
 */

public class ConnectedUser {

	public final static int GIVE_LOGIN  = 0;
	public final static int GIVE_PASSWORD = 1;
	public final static int LOGGED_IN   = 2;
	public final static int DISCONNECTED = 3;


	private ConfigUser user;
	private CommandContext dir;
	private InputStream in;
	private PrintWriter out;
	public String remember;
	public LineBuffer buffer;
	private int state;
	public Object store;
	private int terminalWidth;

	//-----------------------------------------------------------------------
	public ConnectedUser(InputStream in, PrintWriter out, CommandContext dir) {
		this.in   = in;
		this.out  = out;
		this.dir  = dir;
		remember= "";
		buffer = new LineBuffer();
		state = GIVE_LOGIN;
		terminalWidth=78;
	}

	//-----------------------------------------------------------------------
	public void disconnect() {
		try {
			state = DISCONNECTED;
			in.close();
		} catch (IOException e) {
		}
		out.close();
	}

	//-----------------------------------------------------------------------
	public void setDir(CommandContext newDir) {
		dir = newDir;
	}

	//-----------------------------------------------------------------------
	public CommandContext getDir() {
		return dir;
	}

	//-----------------------------------------------------------------------
	public void setState(int newState) {
		state = newState;
	}

	//-----------------------------------------------------------------------
	public int getState() {
		return state;
	}

	//-----------------------------------------------------------------------
	public void setStoredObject(Object obj) {
		store = obj;
	}

	//-----------------------------------------------------------------------
	public Object getStoredObject() {
		return store;
	}

	//-----------------------------------------------------------------------
	public void setUser(ConfigUser user) {
		this.user = user;
	}

	//-----------------------------------------------------------------------
	public ConfigUser getUser() {
		return user;
	}

	//-----------------------------------------------------------------------
	public int getAccessLevel() {
		if (user==null)
			return -1;
		return user.getAccessLevel();
	}

	//------------------------------------------------------
	/**
	 * @return the in
	 */
	public InputStream getIn() {
		return in;
	}

	//------------------------------------------------------
	/**
	 * @return the out
	 */
	public PrintWriter getOut() {
		return out;
	}

	//------------------------------------------------------
	/**
	 * @return the terminalWidth
	 */
	public int getTerminalWidth() {
		return terminalWidth;
	}

	//------------------------------------------------------
	/**
	 * @param terminalWidth the terminalWidth to set
	 */
	public void setTerminalWidth(int terminalWidth) {
		this.terminalWidth = terminalWidth;
	}

	//----------------------------------------------
	/**
	 * @return the buffer
	 */
	public LineBuffer getBuffer() {
		return buffer;
	}


}// ConnectedUser
