package org.prelle.shell;

import java.util.*;

/**
 * PushbackStringTokenizer.java
 *
 *
 * @author  <a href="mailto:prelle@informatik.uni-bremen.de">Stefan Prelle</a>
 * @version $Id: PushbackStringTokenizer.java,v 1.0.0.1 2002/06/21 18:02:52 prelle Exp $
 */

public class PushbackStringTokenizer {

	private List<String> list;
	private String white;

	//--------------------------------------------------
	/**
	 * @param data String to tokenize
	 * @param white Characters that shall be treated as whitespace
	 */
	public PushbackStringTokenizer(String data, String white) {
		list = new ArrayList<String>();
		this.white = white;
		
		StringBuffer tmp = new StringBuffer();
		boolean inQuotes = false;
		for (int i=0; i<data.length(); i++) {
			char c = data.charAt(i);
			if (c=='"') {
				if (inQuotes) {
					list.add(tmp.toString());
					tmp = new StringBuffer();
				}
				inQuotes = !inQuotes;
			} else if (white.indexOf(c)>=0 && !inQuotes) {
				// whitepace
				if (tmp.length()>0) {
					list.add(tmp.toString());
					tmp = new StringBuffer();					
				}
			} else
				tmp.append(c);
		}
		if (tmp.length()>0) 
			list.add(tmp.toString());
		
	}

	//--------------------------------------------------
	/**
	 * Checks if there are tokens left in the stream
	 *
	 * @return TRUE if there are tokens left
	 */
	public boolean hasMoreTokens() {
		return !list.isEmpty();
	}

	//--------------------------------------------------
	/**
	 * Returns the next token of the string
	 *
	 * @return Next token (or <code>null</code>)
	 */
	public String nextToken() {
		if (list.isEmpty())
			return null;

		return (String)list.remove(0).trim();
	}

	//--------------------------------------------------
	/**
	 * Returns a token to the stream. The token will be
	 * returned on the next <code>nextToken</code>-call.
	 *
	 * @param token Token to return
	 */
	public void pushback(String token) {
		list.add(0, token);
	}

	//--------------------------------------------------
	public String toString() {
		StringBuffer buf = new StringBuffer();
		Iterator<String> it = list.iterator();
		while (it.hasNext()) {
			buf.append(it.next());
			if (it.hasNext()) {
				buf.append(white.charAt(0));
			}
		}
		return buf.toString();
	}

	//--------------------------------------------------
	public int getTokensLeft() {
		return list.size();
	}

	//--------------------------------------------------
	public List<String> getTokenList() {
		return list;
	}


}// PushbackStringTokenizer
