package org.prelle.shell;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.PrintWriter;

import org.apache.log4j.Logger;
import org.prelle.user.ConfigUser;

public class StreamShell implements BaseShellListener, Runnable {

	private Logger logger = Logger.getLogger("shell.stream");

	private InputStream in;
	private PrintStream out;
	private CommandContext startPath;
	private BaseShell shell;
	
	private ConnectedUser user;
	private Thread thread;
	
	//----------------------------------------------
	/**
	 * @param in
	 * @param out
	 */
	public StreamShell(InputStream in, PrintStream out, CommandContext startPath) {
		this.in = in;
		this.out= out;
		this.startPath = startPath;
		shell = new BaseShell(this);
		user = new ConnectedUser(in, new PrintWriter(new OutputStreamWriter(out)), startPath);
		
		out.print("> ");
		out.flush();
		
		thread = new Thread(this, "StreamShell");
		thread.start();
	}

	//----------------------------------------------
	/* (non-Javadoc)
	 * @see org.prelle.shell.BaseShellListener#getStartPath(org.prelle.user.ConfigUser)
	 */
	public CommandContext getStartPath(ConfigUser user) {
		return startPath;
	}

	//--------------------------------------------------------------
	public String readResult(ConnectedUser user) throws IOException {
		LineBuffer buffer = user.getBuffer();
		while (true) {
			int dat = in.read();
			if (dat==-1) {
				// CTRL-C
				logger.debug("End of stream");
				out.println("Bye\r");
				out.close();
				return null;				
			} 
//			logger.debug("  dat="+dat);
			if (dat==27) {
				int what = in.read();
				if (what==91) {
					int sub  = in.read();
					// Cursor
					switch (sub) {
					case 65: 
						logger.debug("Cursor up");
						buffer.processKeyEvent(LineBuffer.UP);
						String toDisplay = buffer.toString();
						if (toDisplay!=null) {
							out.print(toDisplay);
							out.flush();							
						}
						break;
					case 66: logger.debug("Cursor down"); break;
					case 67: logger.debug("Cursor right"); break;
					case 68: logger.debug("Cursor left"); break;
					default:
						logger.debug("ESC (91) ("+sub+")");
					}
				} else
					logger.debug("ESC");
			} else if (dat==0) {
				// Follows any CR - ignore
			} else if (dat==3) {
				// CTRL-C
				logger.debug("CTRL-C");
				out.println("Bye\r");
				out.close();
				return null;
			} else if (dat==8) {
				// Delete
				logger.debug("Delete");
				buffer.processKeyEvent(LineBuffer.DELETE);
				out.write(8);
				out.flush();
			} else if (dat==9) {
				// Tabulator
//				buffer.processKeyEvent(LineBuffer.TABULATOR);
				String remem = buffer.getUnfinishedInput();
				user.remember = remem;
				String command = remem+BaseShell.INDICATE_TAB;

				out.flush();
				return command;
			} else if (dat==10) {
				// Return
				buffer.processKeyEvent(LineBuffer.ENTER);
				out.write(13);
				out.write(10);
				out.flush();
				return buffer.getFinishedInput();
//				} else if (dat==26) {
//				// CTRL-Z
////				user.dir = registry.getRoot();
//				user.buf = new StringBuffer();
//				out.write(13);
//				out.write(10);
//				out.flush();
//				return;
			} else if (dat==127) {
				// BackSpace
				logger.debug("DeleteCode with 127");
				buffer.processKeyEvent(LineBuffer.DELETE);
				out.write(8);
				out.flush();
				out.write(32);
				out.write(8);
				out.flush();
			} else if (dat=='?') {
				// Return
				String remem = buffer.getUnfinishedInput();
				user.remember = remem;
				String command = remem+"?";
				out.write(13);
				out.write(10);
				out.flush();
				return command;
			} else {
				buffer.appendChar((char)dat);
				if (!Character.isLetterOrDigit((char)dat)) 
					logger.debug("("+dat+")");
//				out.write((byte)dat);
//				out.flush();
			}
		} // while true
	}

	//----------------------------------------------
	public void run() {

		while (true) {
			try {
				if (in.available()>0) {
					logger.debug("Found data ");
					String line = readResult(user);
					shell.parseLoggedIn(user, line);
				}
			} catch (IOException ioe) {
				out.print(ioe.toString()+"\n\r");
				out.print(shell.getPrompt(user.getDir()));
				out.flush();
			} catch (Exception ioe) {
				ioe.printStackTrace();
				out.print(ioe.toString()+"\n\r");
				out.print(shell.getPrompt(user.getDir()));
				out.flush();
			}

			try {
				Thread.sleep(200);
			} catch (InterruptedException ie) {
				// foo
			}
		}

	}
}
