/**
 * 
 */
package org.prelle.shell;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;
import org.prelle.config.commands.Command;

/**
 * @author prelle
 *
 */
public class TreeCommandContext implements CommandContext {

	private final static Logger logger = Logger.getLogger("shell.tree");
	
	private TreeCommandContext parent;
	private List<TreeCommandContext> children;
	private String name;
	/** Appears in parent listing */
	private String shortHelp;
	/** Precedes children command list */
	private String description;
	private Object node;
	private List<Command> commands;
	
	//------------------------------------------------------
	/**
	 * @parant Parent node or root
	 */
	public TreeCommandContext(TreeCommandContext parent, String name, Object node) {
		this.parent = parent;
		this.name   = name;
		this.node   = node;
		children = new ArrayList<TreeCommandContext>();
		commands = new ArrayList<Command>();
		
//		commands.add(EXIT);
	}

	//------------------------------------------------------
	/* (non-Javadoc)
	 * @see org.prelle.shell.CommandContext#getAvailableCommands()
	 */
	public List<Command> getAvailableCommands() {
		logger.debug("getAvailableCommands");
		return commands;
	}

	//------------------------------------------------------
	public CommandContext getParent() {
		return parent;
	}

	//------------------------------------------------------
	public List<Command> getPossibleCommandCompletions(String input) {
		logger.debug("TODO: getAvailableCommands");
		return null;
	}

	//------------------------------------------------------
	public String toPrompt() {
		if (parent==null)
			return "";
		return parent.toPrompt()+"/"+name;
	}

	//------------------------------------------------------
	public Object getNode() {
		return node;
	}

	//------------------------------------------------------
	public String getName() {
		return name;
	}

	//------------------------------------------------------
	public void addChild(TreeCommandContext child) {
		children.add(child);
		ChangeDirCommand com = new ChangeDirCommand(child);
		commands.add(com);

		ChangeDirCommand exit = new ChangeDirCommand(this, "exit");
		child.addCommand(exit);
		
		Collections.sort(commands, new Comparator<Command>(){
			public int compare(Command o1, Command o2) {
				return o1.getCommandName().compareTo(o2.getCommandName());
			}});
	}

	//------------------------------------------------------
	TreeCommandContext getChildWithName(String name) {
		for (TreeCommandContext ctx : children)
			if (ctx.getName().equals(name))
				return ctx;
		return null;
	}

	//------------------------------------------------------
	/**
	 * @param simpleCommand
	 */
	public void addCommand(Command com) {
		if (!commands.contains(com))
			commands.add(com);
	}

	//------------------------------------------------------
	/**
	 * @return the shortHelp
	 */
	public String getShortHelp() {
		return shortHelp;
	}

	//------------------------------------------------------
	/**
	 * @param shortHelp the shortHelp to set
	 */
	public void setShortHelp(String shortHelp) {
		this.shortHelp = shortHelp;
	}

	//------------------------------------------------------
	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	//------------------------------------------------------
	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

}
