package org.prelle.shell.telnet;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * TCPNetwork.java
 *
 * @author Stefan Prelle
 * @version $Id$
 */

public class TCPNetwork implements Runnable {

	private ServerSocket sock;
	private boolean      stopped;
	private Thread       thread;
	private TCPNetworkListener list;
	private InetAddress  localIP;
	private int          port;

	//---------------------------------------------------------------
	public TCPNetwork(TCPNetworkListener parent, int port) throws IOException {
		sock = new ServerSocket(port);
		this.port = port;
		list = parent;
		localIP = sock.getInetAddress();

		stopped= false;
		thread = new Thread(this);
		thread.setName("TCPNetwork-"+port);
		thread.start();
	}

	//---------------------------------------------------------------
	public TCPNetwork(TCPNetworkListener parent, InetAddress ip, int port) throws IOException {
		sock = new ServerSocket(port, 30, ip); // Bind to socket with backlog 30
		list = parent;
		this.port = port;
		localIP = ip;

		stopped= false;
		thread = new Thread(this);
		thread.setName("TCPNetwork-"+ip+":"+port);
		thread.start();
	}

	//--------------------------------------------------------------
	public InetAddress getLocalIP() {
		return localIP;
	}

	//--------------------------------------------------------------
	public String toString() {
		return "TCPNetwork: "+localIP+":"+port;
	}

	//--------------------------------------------------------------
	public void shutdown() {
		stopped = true;
		thread.interrupt();
	}

	//--------------------------------------------------------------
	public void run() {
		stopped = false;
		while (!stopped) {
			try {
				Socket neu = sock.accept();
				list.newConnection(neu);
			} catch (InterruptedIOException ioe) {
			} catch (IOException ioe) {
				System.err.println("Error on socket: "+ioe);
			}
		}
	}


} // TCPNetwork
