package org.prelle.shell.telnet;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.prelle.shell.BaseShell;
import org.prelle.shell.ConnectedUser;
import org.prelle.shell.LineBuffer;
import org.prelle.user.ConfigUser;
import org.prelle.user.UserBase;

/**
 * TelnetAdmin.java
 *
 *
 * Created: Fri Aug  2 14:28:02 2002
 *
 * @author Stefan Prelle
 * @version $Id$
 */

@SuppressWarnings("static-access")
public class TelnetAdmin implements TCPNetworkListener, Runnable {

	private Logger logger = Logger.getLogger("shell.telnet");

	private final static int SE   = 240;
	//    private final static int NOP  = 241;
	//    private final static int DM   = 242;
	//    private final static int BREAK= 243;
	private final static int IP   = 244;
	//    private final static int AO   = 245;
	//    private final static int AYT  = 246;
	//    private final static int EC   = 247;
	//    private final static int EL   = 248;
	//    private final static int GA   = 249;
	private final static int SB   = 250;
	private final static int WILL = 251;
	private final static int WONT = 252;
	private final static int DO   = 253;
	private final static int DONT = 254;
	private final static int IAC  = 255;

	private final static int SUB_BINARY = 0;
	private final static int SUB_ECHO   = 1;
	private final static int SUB_RECONNECTION = 2;
	private final static int SUB_SUPRESS_GO_AHEAD = 3;
	private final static int SUB_STATUS = 5;
	//    private final static int SUB_TIMING_MARK = 6;
	//    private final static int SUB_OUTPUT_LINE_WIDTH = 8;
	//    private final static int SUB_OUTPUT_PAGE_SIZE = 9;
	//    private final static int SUB_LOGOUT = 18;
	private final static int SUB_TERMTYPE = 24;
	private final static int SUB_NAWS = 31;
	private final static int SUB_LINEMODE = 34;
	//    private final static int SUB_XDISPLOC = 35;
	//    private final static int SUB_EXT_OPT_LIST = 255;

//	private final static char INDICATE_TAB = (char)9;


	private String ip;
	private int port = 4000; 
	private BaseShell shell;
	private UserBase userBase;
	private Thread thread;

	//-----------------------------------------------------------------------
	public TelnetAdmin(BaseShell shell, UserBase ubase, Properties config) {
		this.userBase = ubase;
		this.shell    = shell;
		ip = config.getProperty("telnet.ip");
		port  = Integer.parseInt(config.getProperty("telnet.port", "4001"));

	}

	//-----------------------------------------------------------------------
	public void setIP(String ip) {
		this.ip = ip;
	}

	//-----------------------------------------------------------------------
	public String getIP() {
		return ip;
	}

	//-----------------------------------------------------------------------
	public void setPort(int port) {
		this.port = port;
	}

	//-----------------------------------------------------------------------
	public int getPort() {
		return port;
	}

	//-----------------------------------------------------------------------
	public void start() {
		try {
			TCPNetwork network = null;
			if (ip==null)
				network = new TCPNetwork(this, port);
			else
				network = new TCPNetwork(this, InetAddress.getByName(ip), port);
			thread = new Thread(this);
			thread.start();
			logger.info("Started listening for telnet connections on "+network.getLocalIP().getHostAddress()+":"+port);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}		
	}

	//--------------------------------------------------------------
	void sendWill(OutputStream out, int code) throws IOException {
		byte[] send = new byte[3];
		send[0] = (byte)IAC; 
		send[1] = (byte)WILL; 
		send[2] = (byte)code;  
		out.write(send);
		out.flush();
	}

	//--------------------------------------------------------------
	void sendDo(OutputStream out, int code) throws IOException {
		byte[] send = new byte[3];
		send[0] = (byte)IAC; 
		send[1] = (byte)DO; 
		send[2] = (byte)code;  
		out.write(send);
		out.flush();
	}

	//--------------------------------------------------------------
	private String getCommandName(int com) {
		switch (com) {
		case SE  : return "SE ";
		case IP  : return "IP ";
		case DO  : return "DO ";
		case DONT: return "DON'T ";
		case WILL: return "WILL ";
		case WONT: return "WON'T ";
		case SB  : return "SB ";
		default:
			return "("+com+") ";
		}
	}

	//--------------------------------------------------------------
	public String readResult(ConnectedUser user) throws IOException {
		InputStream in = user.getIn();
		PrintWriter out = user.getOut();
		LineBuffer buffer = user.buffer;

		while (true) {
			int dat = in.read();
			if (dat==255) {
				logger.debug("IAC ");
				// Negotiation
				int next = in.read();
				logger.debug(getCommandName(next));
				int sub = in.read();
				switch (sub) {
				case SUB_BINARY: logger.debug("BINARY"); break;
				case SUB_ECHO: logger.debug("ECHO"); break;
				case SUB_RECONNECTION: logger.debug("RECONNECTION"); break;
				case SUB_SUPRESS_GO_AHEAD: logger.debug("SUPRESS GO AHEAD"); break;
				case SUB_STATUS: logger.debug("STATUS"); break;
				case SUB_LINEMODE: logger.debug("LINEMODE"); break;
				case SUB_TERMTYPE: logger.debug("TERMTYPE"); break;
				case SUB_NAWS: logger.debug("NAWS"); break;
				default:
					logger.debug("("+sub+") ");
				}

				// Sub-Negotiation
				if (next==SB) {
					if (sub==SUB_NAWS) {
						// NAWS Sub negotiation
						int x1 = in.read();
						int x2 = in.read();
						int y1 = in.read();
						int y2 = in.read();
						int x = x1*256 + x2;
						int y = y1*256 + y2;
						logger.debug( x+"*"+y);
						user.setTerminalWidth(x);
					}
					int first = in.read();
					int sec   = in.read();
					if (first!=IAC || sec!=SE) 
						logger.error("Awaited SE but received: "+first+" "+sec);
				}
				return null;
			} else if (dat==27) {
				int what = in.read();
				if (what==91) {
					int sub  = in.read();
					// Cursor
					switch (sub) {
					case 65: 
						logger.debug("Cursor up");
						buffer.processKeyEvent(LineBuffer.UP);
						String toDisplay = buffer.toString();
						if (toDisplay!=null) {
							out.print(toDisplay);
							out.flush();							
						}
						break;
					case 66: logger.debug("Cursor down"); break;
					case 67: logger.debug("Cursor right"); break;
					case 68: logger.debug("Cursor left"); break;
					default:
						logger.debug("ESC (91) ("+sub+")");
					}
				} else
					logger.debug("ESC");
			} else if (dat==0) {
				// Follows any CR - ignore
			} else if (dat==3) {
				// CTRL-C
				logger.debug("CTRL-C");
				out.println("Bye\r");
				out.close();
				return null;
			} else if (dat==8) {
				// Delete
				logger.debug("Delete");
				buffer.processKeyEvent(LineBuffer.DELETE);
				out.write(8);
				out.flush();
			} else if (dat==9) {
				// Tabulator
//				buffer.processKeyEvent(LineBuffer.TABULATOR);
				String remem = buffer.getUnfinishedInput();
				user.remember = remem;
				String command = remem+BaseShell.INDICATE_TAB;

				out.flush();
				return command;
			} else if (dat==13) {
				// Return
				buffer.processKeyEvent(LineBuffer.ENTER);
				out.write(13);
				out.write(10);
				out.flush();
				return buffer.getFinishedInput();
//				} else if (dat==26) {
//				// CTRL-Z
////				user.dir = registry.getRoot();
//				user.buf = new StringBuffer();
//				out.write(13);
//				out.write(10);
//				out.flush();
//				return;
			} else if (dat==127) {
				// BackSpace
				logger.debug("DeleteCode with 127");
				buffer.processKeyEvent(LineBuffer.DELETE);
				out.write(8);
				out.flush();
				out.write(32);
				out.write(8);
				out.flush();
			} else if (dat=='?') {
				// Return
				String remem = buffer.getUnfinishedInput();
				user.remember = remem;
				String command = remem+"?";
				out.write(13);
				out.write(10);
				out.flush();
				return command;
			} else {
				buffer.appendChar((char)dat);
				if (!Character.isLetterOrDigit((char)dat)) 
					logger.debug("("+dat+")");
				out.write((byte)dat);
				out.flush();
			}
		} // while true
	}

	//--------------------------------------------------------------
	public void newConnection(Socket sock) {
		logger.info("New connection from "+sock.getInetAddress().getHostAddress());

		// Telnet RFC 0854
		try {

			InputStream  in = sock.getInputStream();
			PrintWriter out = new PrintWriter(new OutputStreamWriter(sock.getOutputStream()));

			ConnectedUser user = shell.createNewUserConnection(in, out);

			sendWill(sock.getOutputStream(), SUB_ECHO);
			readResult(user);
			sendWill(sock.getOutputStream(), SUB_SUPRESS_GO_AHEAD);
			readResult(user);
			sendDo(sock.getOutputStream(), SUB_TERMTYPE);
			readResult(user);
			sendDo(sock.getOutputStream(), SUB_NAWS);
			readResult(user);

			out.print("Login: ");
			out.flush();
			//      	    String prompt = conf.getPrompt(user.dir, user.lvl);
			//      	    out.print(prompt);
			//      	    out.flush();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	//--------------------------------------------------------------
	private void parseGiveLogin(ConnectedUser user, String line) {
		if (line==null) 
			return;
		logger.debug("parseGiveLogin("+line+")");
		if (line.length()==0) {
			user.getOut().print("Login: ");
			user.getOut().flush();
			return;
		}

		// Remember login
		user.setStoredObject(line);
		user.setState(ConnectedUser.GIVE_PASSWORD);

		user.getOut().print("Password: ");
		user.getOut().flush();
	}

	//--------------------------------------------------------------
	private void parseGivePassword(ConnectedUser user, String line) {
		if (line==null) 
			return;
		logger.debug("parseGivePassword("+line+")");
		if (line.length()==0) {
			user.getOut().print("Password: ");
			user.getOut().flush();
			return;
		}

		String login = (String)user.getStoredObject();
		ConfigUser account = userBase.getUserWithLogin(login);
		if (account==null || !account.matchesPassword(line)) {
			if (account==null) {
				logger.warn("Invalid login '"+login+"'");
			} else
				logger.warn("Invalid password for login '"+login+"'");
			user.setState(ConnectedUser.GIVE_LOGIN);
			user.getOut().println("Invalid login or password.\r\n");
			user.getOut().flush();
			user.getOut().print("Login: ");
			user.getOut().flush();
			return;
		}

		user.setUser(account);
		user.setDir(shell.getInitialContext(user.getUser()));
		user.setState(ConnectedUser.LOGGED_IN);

		logger.info(account.getName()+" logged in.");
		user.getOut().print("You are logged in as "+account.getName()+"\r\n");
		user.getOut().print(shell.getPrompt(user.getDir()));
		user.getOut().flush();
	}

	//--------------------------------------------------------------
	public void run() {
		logger.debug("Start iterating through known connections");
		
		List<ConnectedUser> toRemove = new ArrayList<ConnectedUser>();
		
		while (true) {
			for (ConnectedUser user : shell.getConnectedUser()) {
				try {
					if (user.getIn().available()>0) {
						logger.debug("Found data for user "+user);
						String line = readResult(user);
						switch (user.getState()) {
						case ConnectedUser.GIVE_LOGIN:
							parseGiveLogin(user, line);
							break;
						case ConnectedUser.GIVE_PASSWORD:
							parseGivePassword(user, line);
							break;
						case ConnectedUser.LOGGED_IN:
							shell.parseLoggedIn(user, line);
							break;
						}
					}
				} catch (IOException ioe) {
					shell.removeUser(user);
					user.getOut().print(ioe.toString()+"\n\r");
					user.getOut().print(shell.getPrompt(user.getDir()));
					user.getOut().flush();
				} catch (Exception ioe) {
					ioe.printStackTrace();
					user.getOut().print(ioe.toString()+"\n\r");
					user.getOut().print(shell.getPrompt(user.getDir()));
					user.getOut().flush();
				}
				
				if (user.getState()==ConnectedUser.DISCONNECTED)
					toRemove.add(user);
			} // while e.hasMoreElements
			
			// Remove disconnected user
			for (ConnectedUser user : toRemove)
				shell.removeUser(user);
			
			try {
				Thread.sleep(50);
			} catch (InterruptedException ie) {
			}
		}
	}	    

}// TelnetAdmin
